package com.chelvc.framework.security.context;

import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.AESUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

/**
 * 安全处理上下文工具类
 * <p>
 * 角色资源授权信息从Nacos配置中心获取并动态更新
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class SecurityContextHolder {
    /**
     * 加解密字符串特征前缀
     */
    public static final String CODEC_CHARACTERISTIC_PREFIX = "ENC(";

    /**
     * 加解密字符串特征后缀
     */
    public static final String CODEC_CHARACTERISTIC_SUFFIX = ")";

    /**
     * 权限属性名称
     */
    public static final String PERMISSION_PROPERTY = "platform.permission";

    /**
     * 方法资源拆分匹配模式
     */
    private static final Pattern METHOD_RESOURCE_SPLIT_PATTERN = StringUtils.getPattern(" +");

    /**
     * 权限类型引用
     */
    private static final TypeReference<Map<String, Set<String>>> PERMISSION_TYPE_REFERENCE =
            new TypeReference<Map<String, Set<String>>>() {
            };

    /**
     * 数据加密
     *
     * @param plaintext 数据明文
     * @return 数据密文
     */
    public static String encrypt(String plaintext) {
        if (StringUtils.isEmpty(plaintext)) {
            return plaintext;
        }
        SecurityProperties properties = ApplicationContextHolder.getBean(SecurityProperties.class);
        String secret = Objects.requireNonNull(properties.getSecret(), "secret invalid");
        String device = Objects.requireNonNull(SessionContextHolder.getDevice(), "device invalid");
        String iv = StringUtils.substring(device, 0, 16);
        String ciphertext = AESUtils.encode(plaintext, secret, iv);
        if (log.isDebugEnabled()) {
            log.debug("Data encrypt: {} -> {}", plaintext, ciphertext);
        }
        return CODEC_CHARACTERISTIC_PREFIX + ciphertext + CODEC_CHARACTERISTIC_SUFFIX;
    }

    /**
     * 数据加密
     *
     * @param plaintext 数据明文
     * @param safety    是否使用安全模式（如果加密失败则返回明文）
     * @return 数据密文
     */
    public static String encrypt(String plaintext, boolean safety) {
        // 非安全模式
        if (!safety) {
            return encrypt(plaintext);
        }

        // 安全模式
        try {
            return encrypt(plaintext);
        } catch (Exception e) {
            // 如果加密失败则返回明文
            log.error("Data encrypt failed: {}", e.getMessage());
            return plaintext;
        }
    }

    /**
     * 数据解密
     *
     * @param ciphertext 数据密文
     * @return 数据明文
     */
    public static String decrypt(String ciphertext) {
        if (StringUtils.isEmpty(ciphertext) || !(ciphertext.startsWith(CODEC_CHARACTERISTIC_PREFIX)
                && ciphertext.endsWith(CODEC_CHARACTERISTIC_SUFFIX))) {
            return ciphertext;
        }
        SecurityProperties properties = ApplicationContextHolder.getBean(SecurityProperties.class);
        String secret = Objects.requireNonNull(properties.getSecret(), "secret invalid");
        String device = Objects.requireNonNull(SessionContextHolder.getDevice(), "device invalid");
        String iv = StringUtils.substring(device, 0, 16);
        ciphertext = ciphertext.substring(
                CODEC_CHARACTERISTIC_PREFIX.length(), ciphertext.length() - CODEC_CHARACTERISTIC_SUFFIX.length()
        );
        String plaintext = AESUtils.decode(ciphertext, secret, iv);
        if (log.isDebugEnabled()) {
            log.debug("Data decrypt: {} -> {}", ciphertext, plaintext);
        }
        return plaintext;
    }

    /**
     * 判断当前用户是否拥有指定资源权限
     *
     * @param method 请求方式
     * @param uri    资源地址
     * @return true/false
     */
    public static boolean hasPermission(String method, String uri) {
        if (StringUtils.isEmpty(method) || StringUtils.isEmpty(uri)) {
            return false;
        }

        // 基于用户角色及对应的资源判断是否拥有权限
        String[] roles = StringUtils.splitActives(SessionContextHolder.getAuthority(false));
        if (roles == null || roles.length == 0) {
            return false;
        }
        Map<String, Set<String>> permissions = ApplicationContextHolder.getSafeProperty(
                PERMISSION_PROPERTY, PERMISSION_TYPE_REFERENCE
        );
        if (CollectionUtils.isEmpty(permissions)) {
            return false;
        }
        return Stream.of(roles).anyMatch(role -> {
            Set<String> resources = permissions.get(role);
            return !CollectionUtils.isEmpty(resources) && resources.stream().anyMatch(resource -> {
                if (StringUtils.isEmpty(resource) || (resource = resource.trim()).isEmpty()) {
                    return false;
                }
                String[] sections = METHOD_RESOURCE_SPLIT_PATTERN.split(resource);
                if (sections.length == 1) {
                    return StringUtils.isPath(uri, sections[0]);
                }
                return Objects.equals(method, sections[0]) && StringUtils.isPath(uri, sections[1]);
            });
        });
    }
}
