package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.chelvc.framework.base.util.DesensitizeUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.annotation.Desensitize;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

/**
 * 敏感数据脱敏序列化处理器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class JacksonDesensitizeSerializer extends StdSerializer<String> implements ContextualSerializer {
    /**
     * 敏感数据脱敏表达式/脱敏模型映射表
     */
    private static final Map<String, DesensitizeUtils.Mode> DESENSITIZE_MODE_MAPPING = new ConcurrentHashMap<>();

    /**
     * 数据脱敏模式
     */
    private final DesensitizeUtils.Mode mode;

    public JacksonDesensitizeSerializer() {
        this(null);
    }

    public JacksonDesensitizeSerializer(DesensitizeUtils.Mode mode) {
        super(String.class);
        this.mode = mode;
    }

    /**
     * 根据脱敏表达式获取敏感数据脱敏模型
     *
     * @param expression 数据脱敏表达式
     * @return 数据脱敏模型
     */
    private DesensitizeUtils.Mode expression2mode(String expression) {
        return DESENSITIZE_MODE_MAPPING.computeIfAbsent(expression, k -> DesensitizeUtils.analyse(expression));
    }

    /**
     * 获取脱敏表达式
     *
     * @param property 对象属性
     * @return 脱敏表达式
     */
    private String getDesensitizeExpression(BeanProperty property) {
        Desensitize desensitize = property.getAnnotation(Desensitize.class);
        return desensitize == null ? null : StringUtils.replace(desensitize.expression(), " ", StringUtils.EMPTY);
    }

    @Override
    public void serialize(String plaintext, JsonGenerator generator, SerializerProvider serializer)
            throws IOException {
        if (this.mode == null || StringUtils.isEmpty(plaintext)) {
            generator.writeString(plaintext);
        } else {
            String ciphertext = this.mode.desensitize(plaintext);
            if (log.isDebugEnabled()) {
                log.debug("Data desensitize: {} -> {}", plaintext, ciphertext);
            }
            generator.writeString(ciphertext);
        }
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider serializer, BeanProperty property) {
        if (property == null) {
            return serializer.getDefaultNullValueSerializer();
        } else if (property.getType().getRawClass() != String.class) {
            return this;
        }
        String expression = this.getDesensitizeExpression(property);
        Assert.isTrue(!StringUtils.isEmpty(expression), "Desensitize expression must not be empty");
        return new JacksonDesensitizeSerializer(this.expression2mode(expression));
    }
}
