package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import java.util.Objects;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.CodecUtils;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.ObjectUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

/**
 * 数字签名校验拦截器
 *
 * @author Woody
 * @date 2023/8/3
 */
@Slf4j
@Order(20)
@Component
public class SignatureValidateInterceptor implements Filter {
    /**
     * 数字签名校验
     *
     * @param request  Http请求对象
     * @param response Http相应对象
     * @return true/false
     * @throws IOException I/O操作异常
     */
    private boolean validate(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 判断请求是否需要放行
        SecurityProperties properties = ApplicationContextHolder.getBean(SecurityProperties.class);
        SecurityProperties.Signature config = properties.getSignature();
        if (ObjectUtils.isEmpty(config.getPermit()) && ObjectUtils.isEmpty(config.getRequire())) {
            return true;
        }
        String uri = HttpUtils.getRequestURI(request);
        if (StringUtils.isPath(uri, config.getPermit()) || (ObjectUtils.nonEmpty(config.getRequire())
                && !StringUtils.isPath(uri, config.getRequire()))) {
            return true;
        }

        // 数字签名校验
        String device = request.getHeader(SessionContextHolder.HEADER_DEVICE);
        String platform = request.getHeader(SessionContextHolder.HEADER_PLATFORM);
        String terminal = request.getHeader(SessionContextHolder.HEADER_TERMINAL);
        String version = request.getHeader(SessionContextHolder.HEADER_VERSION);
        String timestamp = request.getHeader(SessionContextHolder.HEADER_TIMESTAMP);
        String secret = Objects.requireNonNull(properties.getSecret(), "secret invalid");
        String payload = HttpUtils.serialize(request, true);
        String ciphertext = CodecUtils.md5(payload + secret + device + platform + terminal + version + timestamp);
        if (log.isDebugEnabled()) {
            log.debug("Data signature: {} -> {}", payload, ciphertext);
        }
        if (Objects.equals(ciphertext, request.getHeader(SessionContextHolder.HEADER_SIGNATURE))) {
            return true;
        }

        // 签名无效
        log.warn(HttpUtils.getExceptionMessage(request, HttpStatus.FORBIDDEN, "Signature invalid"));
        if (StringUtils.isPath(uri, config.getIgnore())) {
            return true;
        }
        response.setStatus(HttpStatus.FORBIDDEN.value());
        HttpUtils.writeUnifiedResponse(response, Result.build(HttpStatus.FORBIDDEN));
        return false;
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (this.validate((HttpServletRequest) request, (HttpServletResponse) response)) {
            chain.doFilter(request, response);
        }
    }
}
